# -*- coding: utf-8 -*-

"""\
Wrappers for multiple types of Discord related files.

Source: https://github.com/cmyui/cmyui_pkg

:copyright: (c) 2020 cmyui
:license: MIT, see LICENSE for details.
"""

from .webhook import *
